///File : ConfigObj.hpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#if not defined CONFIG_OBJ_HPP_MAXI_SHOOT
#define CONFIG_OBJ_HPP_MAXI_SHOOT
//------------------------------------------------------------------------------
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>

#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm> // Sort

#if defined WIN32
    #include <windows.h>
#else
    #include <limits.h> // Permet d'utiliser PATH_MAX, longeur max d'un path.
#endif
//------------------------------------------------------------------------------
#include "myConf.hpp"
#include "RcManager.hpp"
//------------------------------------------------------------------------------

#define NB_SCORE_SAVED 10
#define LINUX_SCORE_FILE_DIR "/var/games"
//------------------------------------------------------------------------------

struct StructScore
{
    std::string user;
    int score;
};

// CompareScore
bool compareScore(const StructScore &i, const StructScore &j);

enum{GAME_REQUEST_NEW, GAME_PLAY};

class ConfigObj
{
public:
    void LoadConfig();
    void LoadMode();
    void SaveAll();

    bool IsFpsOn() const;
    void SetFpsOn(const bool &fps);

    bool IsFullScreenOn() const;
    void SetFullScreenOn(const bool &fullscreen);

    std::string GetCurrentVideoModeText() const;
    void ChangeVideoMode();
    sf::VideoMode GetCurrentVideoMode() const;

    int GetGameState() const;
    void SetGameState(const int &state);

    std::string GetDataDir();
    std::string GetUserConfigDir();
    void FindStandardPaths();

    void LoadModeDirNamesList();
    std::vector<std::string> GetModeDirNamesList();
    void SetCurrentModeDirName(const std::string &modeDirName);
    std::string GetTempModeTotalDir(const std::string &tmpModeName);
    std::string GetCurrentModeDirName();
    std::string GetNextMode(const std::string &modeDirName);

    void SaveScores();
    void LoadScores();
    bool AddScore(const int &score);
    std::string GetScores();

    RcManager *GetRcManager();

private:
    std::string m_modeDirName;

    std::string m_dataDir;
    std::string m_userConfigDir;

    StructScore m_scoresTbl[NB_SCORE_SAVED];

    std::vector<std::string> m_modeDirNamesList;

    int m_gameState;
    bool m_fpsOn;
    bool m_fullScreenOn;

    sf::VideoMode m_currentVideoMode;

    RcManager m_RcManager;
};
//------------------------------------------------------------------------------

#endif // CONFIG_OBJ_HPP_MAXI_SHOOT
