///File : GameScreen.cpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "GameScreen.hpp"
//------------------------------------------------------------------------------

void GameScreen::Shoot(const int &gunId, const bool &isLaser)
{
    int simultaneousBullet;
    if(isLaser) // Son pr laser géré dans le Run
    {
        simultaneousBullet = 1;
    }
    else
    {
        m_RcManager->PlaySnd("snd-shot");
        simultaneousBullet = m_Guns[gunId].m_simultaneousBullet;
    }

    for(int i = 0; i < simultaneousBullet; i++)
    {
        StructBullet bullet;
        bullet.isLaser = isLaser;

        if(isLaser)
        {
            bullet.spr = sf::Sprite(m_RcManager->GetImage("laser"));
            bullet.spr.SetCenter(bullet.spr.GetImage()->GetWidth() / 2, bullet.spr.GetImage()->GetHeight() / 2);
        }
        else
        {
            bullet.spr= sf::Sprite(m_RcManager->GetImage("bullet"));
            bullet.spr.SetCenter(bullet.spr.GetImage()->GetWidth() / 2, bullet.spr.GetImage()->GetHeight() / 2);
        }

        if(simultaneousBullet == 1)
        {
            bullet.spr.SetRotation(m_Guns[gunId].GunSpr.GetRotation());
        }
        else
        {
            bullet.spr.SetRotation(sf::Randomizer::Random(m_Guns[gunId].GunSpr.GetRotation() - float(simultaneousBullet), m_Guns[gunId].GunSpr.GetRotation() + float(simultaneousBullet)));
        }
        bullet.spr.SetPosition(m_Guns[gunId].GunSpr.GetPosition());

        float angleRad = (m_Guns[gunId].GunSpr.GetRotation() - 90) * DEG_TO_RAD;
        bullet.spr.SetPosition(bullet.spr.GetPosition().x - cos(angleRad) * m_RcManager->GetImage("gun").GetHeight(), bullet.spr.GetPosition().y + sin(angleRad) * m_RcManager->GetImage("gun").GetHeight());

        m_BulletsList.push_back(bullet);
    }
}
//------------------------------------------------------------------------------

void GameScreen::BulletMove(const float &frameTime)
{
    for(int i = 0; i < m_BulletsList.size(); i++)
    {
        float angleRad = (m_BulletsList[i].spr.GetRotation() - 90) * DEG_TO_RAD;

        if(!m_BulletsList[i].isLaser)
        {
            m_BulletsList[i].spr.SetPosition(m_BulletsList[i].spr.GetPosition().x - cos(angleRad) * m_RcManager->GetValue("BULLET_SPEED") * frameTime, m_BulletsList[i].spr.GetPosition().y + sin(angleRad) * m_RcManager->GetValue("BULLET_SPEED") * frameTime);
        }
        else
        {
            m_BulletsList[i].spr.SetPosition(m_BulletsList[i].spr.GetPosition().x - cos(angleRad) * m_RcManager->GetValue("LASER_SPEED") * frameTime, m_BulletsList[i].spr.GetPosition().y + sin(angleRad) * m_RcManager->GetValue("LASER_SPEED") * frameTime);
        }

        if(m_BulletsList[i].spr.GetPosition().x < -ITEMS_ERASE_BORDER || m_BulletsList[i].spr.GetPosition().x > m_RcManager->GetValue("GAME_SCREEN_WIDTH") + ITEMS_ERASE_BORDER || m_BulletsList[i].spr.GetPosition().y < -ITEMS_ERASE_BORDER || m_BulletsList[i].spr.GetPosition().y > m_RcManager->GetValue("GAME_SCREEN_HEIGHT") + ITEMS_ERASE_BORDER)
        {
            // Delete if out of the screen
            m_BulletsList.erase(m_BulletsList.begin() + i);
            continue;
        }
    }
}
//------------------------------------------------------------------------------
void GameScreen::NewGun(const float &gunXPos, const bool &isHuman)
{
    StructGun gun;
    gun.GunSpr.SetImage(m_RcManager->GetImage("gun"));
    gun.GunSpr.SetCenter(gun.GunSpr.GetImage()->GetWidth() / 2, gun.GunSpr.GetImage()->GetHeight());
    gun.GunSpr.SetPosition(gunXPos, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 130); // Attention : sprite center has moved !
    gun.GunSpr.SetRotation(0); // Attention : sprite center has moved !

    gun.BunkerSpr.SetImage(m_RcManager->GetImage("bunker"));
    gun.BunkerSpr.SetPosition(gunXPos - gun.BunkerSpr.GetImage()->GetWidth() / 2, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - gun.BunkerSpr.GetImage()->GetHeight());

    gun.m_isHuman = isHuman;

    gun.m_timeBeforeLastShot = m_RcManager->GetValue("TIME_BEFORE_NEXT_SHOT"); // Pr pouvoir tirer tout de suite.
    gun.m_simultaneousBullet = 1;

    gun.m_timeBeforeLastLaser = m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER"); // Pr pouvoir tirer tout de suite.
    gun.m_timeBeforeLastLaserBullet = m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER_BULLET"); // Pr pouvoir tirer tout de suite.
    gun.m_laserBulletLeft = 0;

    gun.isSight = false;

    m_Guns.push_back(gun);
}
//------------------------------------------------------------------------------
void GameScreen::NewGame()
{
    m_Enemies.clear();
    m_BulletsList.clear();
    m_Debris.clear();
    m_Guns.clear();

    m_RcManager->NewGame();

    m_shieldAlpha = m_RcManager->GetValue("MIN_SHIELD_ALPHA");

    m_RcManager->SetValue("SIMULTANEOUS_BULLET", 1);
    m_RcManager->SetValue("FRIENDS_NB", 0);
    m_RcManager->SetValue("IS_SIGHT", 0);

    m_scoreSent = false;
    m_haveScore = false;

    if(m_ConfigObj->GetCurrentModeDirName() != "Test")
    {
        NewGun(m_RcManager->GetValue("GAME_SCREEN_WIDTH") / 2);
    }
    else
    {
        NewGun(m_RcManager->GetValue("GAME_SCREEN_WIDTH") / 2, false);
    }
}
//------------------------------------------------------------------------------
void GameScreen::CreateExplosion(const StructBurstInfo &burstInfo)
{
    StructDebris debris;

    if(!burstInfo.isLife)
    {
        debris.lifeTime = debris.lifeTimeLeft = m_RcManager->GetValue("BURST_IMG_LIFE_TIME");
        debris.speed.x = 0;
        debris.speed.y = 0;
        debris.spr.SetImage(m_RcManager->GetImage("img-enemy-burst"));
        sf::Vector2<float> debrisPos(burstInfo.pos.x - debris.spr.GetImage()->GetWidth() / 2, burstInfo.pos.y - debris.spr.GetImage()->GetHeight() / 2);
        debris.spr.SetPosition(debrisPos);
        m_Debris.push_back(debris);

        int debrisNb = burstInfo.isMini ? m_RcManager->GetValue("DEBRIS_NB") / 3 : m_RcManager->GetValue("DEBRIS_NB");

        int maxDebrisId;
        for(int i = 1; m_RcManager->ImageExists(std::string("img-enemy-debris(") + intToString(i) + std::string(")")); i++)
        {
            maxDebrisId = i;
        }

        for(int i = 0; i < debrisNb; i++)
        {
            debris.lifeTime = debris.lifeTimeLeft = sf::Randomizer::Random(m_RcManager->GetValue("DEBRIS_LIFE_TIME_MIN"), m_RcManager->GetValue("DEBRIS_LIFE_TIME_MAX"));
            float angleRad;

            angleRad = -(sf::Randomizer::Random(burstInfo.angleMin, burstInfo.angleMax) + 90) * DEG_TO_RAD;
            debris.speed.x = cos(angleRad) * sf::Randomizer::Random(m_RcManager->GetValue("DEBRIS_SPEED_MIN"), m_RcManager->GetValue("DEBRIS_SPEED_MAX"));
            debris.speed.y = sin(angleRad) * sf::Randomizer::Random(m_RcManager->GetValue("DEBRIS_SPEED_MIN"), m_RcManager->GetValue("DEBRIS_SPEED_MAX"));
            debris.spr.SetImage(m_RcManager->GetImage(std::string("img-enemy-debris(") + intToString(sf::Randomizer::Random(1, maxDebrisId)) + std::string(")")));
            debris.spr.SetPosition(burstInfo.pos.x - debris.spr.GetImage()->GetWidth() / 2, burstInfo.pos.y - debris.spr.GetImage()->GetHeight() / 2);
            m_Debris.push_back(debris);
        }
    }

    if(burstInfo.isOnShield)
    {
        m_shieldAlpha = 255;

        debris.lifeTime = debris.lifeTimeLeft = 2;
        debris.speed.x = 0;
        debris.speed.y = 0;
        debris.spr.SetImage(m_RcManager->GetImage("shield-impact"));
        debris.spr.SetPosition(burstInfo.pos.x - debris.spr.GetImage()->GetWidth() / 2, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - m_RcManager->GetImage("shield").GetHeight());
        m_Debris.push_back(debris);
    }
}
//------------------------------------------------------------------------------
void GameScreen::EnemiesMove(const float &frameTime)
{
    for(int i = 0; i < m_Enemies.size(); i++)
    {
        bool deleted = false;
        // Move
        m_Enemies[i].spr.SetPosition(m_Enemies[i].spr.GetPosition().x + m_Enemies[i].speed.x * frameTime, m_Enemies[i].spr.GetPosition().y + m_Enemies[i].speed.y * frameTime);

        // Delete if out of the screen
        if(m_Enemies[i].spr.GetPosition().x < -ITEMS_ERASE_BORDER || m_Enemies[i].spr.GetPosition().x > m_RcManager->GetValue("GAME_SCREEN_WIDTH") + ITEMS_ERASE_BORDER || m_Enemies[i].spr.GetPosition().y < -ITEMS_ERASE_BORDER || m_Enemies[i].spr.GetPosition().y > m_RcManager->GetValue("GAME_SCREEN_HEIGHT") + ITEMS_ERASE_BORDER)
        {
            m_Enemies.erase(m_Enemies.begin() + i);
            continue; // Important : pour que l'enemie ne soit pas supprimé deux fois !
        }

        sf::Rect<float> enemyRect(m_Enemies[i].spr.GetPosition().x, m_Enemies[i].spr.GetPosition().y, m_Enemies[i].spr.GetPosition().x + m_Enemies[i].spr.GetImage()->GetWidth(), m_Enemies[i].spr.GetPosition().y + m_Enemies[i].spr.GetImage()->GetHeight());
        sf::Vector2<float> burstPos(m_Enemies[i].spr.GetPosition().x + m_Enemies[i].spr.GetImage()->GetWidth() / 2, m_Enemies[i].spr.GetPosition().y + m_Enemies[i].spr.GetImage()->GetHeight() / 2); //Center of the enemy
        // Delete if shot
        for(int j = 0; j < m_BulletsList.size(); j++)
        {
            sf::Rect<float> bulletRect;
            if(m_BulletsList[i].isLaser)
            {
                bulletRect = sf::Rect<float>(m_BulletsList[j].spr.GetPosition().x, m_BulletsList[j].spr.GetPosition().y, m_BulletsList[j].spr.GetPosition().x + m_RcManager->GetImage("bullet").GetWidth(), m_BulletsList[j].spr.GetPosition().y + m_RcManager->GetImage("bullet").GetHeight());
            }
            else
            {
                bulletRect = sf::Rect<float>(m_BulletsList[j].spr.GetPosition().x, m_BulletsList[j].spr.GetPosition().y, m_BulletsList[j].spr.GetPosition().x + m_RcManager->GetImage("laser").GetWidth(), m_BulletsList[j].spr.GetPosition().y + m_RcManager->GetImage("laser").GetHeight());
            }


            if(enemyRect.Intersects(bulletRect)) // Si les 2 rectangles se touchent (bullet + enemy)
            {
                float bulletRotation = m_BulletsList[j].spr.GetRotation();
                m_BulletsList.erase(m_BulletsList.begin() + j);

                m_Enemies[i].life -= m_RcManager->GetValue("BULLET_DOMAGES");

                if(m_Enemies[i].life <= 0)
                {
                    if(m_Enemies[i].specialSkill.find("present") != -1)
                    {
                        for(int k = 0; k < 5 + m_Guns[0].m_simultaneousBullet; k++)
                        {
                            StructEnemy enemy;
                            enemy = m_RcManager->GetEnemyFromId("1");
                            enemy.spr.SetPosition(burstPos);
                            m_Enemies.push_back(enemy);
                        }
                    }
                    m_RcManager->PlaySnd(m_Enemies[i].sndBurst);
                    if(m_Enemies[i].specialSkill.find("no-burst") != -1)
                    {
                        StructDebris debris;
                        debris.lifeTime = debris.lifeTimeLeft = m_RcManager->GetValue("NO_BURST_SHADOW_LIFE_TIME");
                        debris.speed = sf::Vector2f(m_Enemies[i].speed.x * m_RcManager->GetValue("NO_BURST_SHADOW_SPEED"), m_Enemies[i].speed.y * m_RcManager->GetValue("NO_BURST_SHADOW_SPEED"));
                        debris.spr = m_Enemies[i].spr;
                        m_Debris.push_back(debris);
                    }
                    else
                    {
                        StructBurstInfo tmpStruct = {burstPos, bulletRotation - m_RcManager->GetValue("DEBRIS_ENEMY_BURST_ANGLE") / 2, bulletRotation + m_RcManager->GetValue("DEBRIS_ENEMY_BURST_ANGLE") / 2, false, false, false};
                        CreateExplosion(tmpStruct);
                    }
                    m_RcManager->SetValue("SCORE", m_RcManager->GetValue("SCORE") + 1);

                    m_Enemies.erase(m_Enemies.begin() + i);
                    deleted = true; // Important : pour que la boucle soit arrétée juste aprés.
                    // Attention : On est dans la boucle des balles !
                    break; // Important : pour que l'enemie ne soit pas supprimé deux fois : Si plusieurs balles touchent un enemie, il sera supprimé deux fois.
                }
                else
                {
                    m_RcManager->PlaySnd(m_Enemies[i].sndResist);

                    if(m_Enemies[i].imageResistAlias != "no")
                    {
                        StructDebris debris;
                        debris.lifeTime = debris.lifeTimeLeft = m_RcManager->GetValue("IMG_RESIST_LIFE_TIME");
                        debris.speed = m_Enemies[i].speed;
                        debris.spr.SetImage(m_RcManager->GetImage(m_Enemies[i].imageResistAlias));
                        debris.spr.SetPosition(m_Enemies[i].spr.GetPosition().x + m_Enemies[i].imageResistPos.x, m_Enemies[i].spr.GetPosition().y + m_Enemies[i].imageResistPos.y);
                        m_Debris.push_back(debris);
                    }
                }
            }
        }

        if(deleted)
        {
            continue;
        }

        // Delete if shok the shield
        if(float(m_RcManager->GetValue("GAME_SCREEN_HEIGHT")) - float(m_RcManager->GetImage("shield").GetHeight()) <= m_Enemies[i].spr.GetPosition().y + m_Enemies[i].spr.GetImage()->GetHeight())
        {
            m_RcManager->SetValue("LIFE", m_RcManager->GetValue("LIFE") - m_Enemies[i].domages);
            m_RcManager->PlaySnd(m_Enemies[i].sndShieldShock);

            StructBurstInfo tmpStruct = {burstPos, -m_RcManager->GetValue("SHIELD_BURST_DEBRIS_ANGLE") / 2, m_RcManager->GetValue("SHIELD_BURST_DEBRIS_ANGLE") / 2, true, m_Enemies[i].domages <= 0, true};
            CreateExplosion(tmpStruct);

            if(m_Enemies[i].specialSkill.find("more-bullet-box") != -1)
            {
                m_Guns[0].m_simultaneousBullet++;
                m_RcManager->SetValue("SIMULTANEOUS_BULLET", m_Guns[0].m_simultaneousBullet);
            }
            else if(m_Enemies[i].specialSkill.find("sight-box") != -1)
            {
                m_Guns[0].isSight = true;
                m_RcManager->SetValue("IS_SIGHT", m_Guns[0].isSight);
            }
            else if(m_Enemies[i].specialSkill.find("friend-box-one") != -1)
            {
                if(m_Guns.size() == 1)
                {
                    NewGun(m_RcManager->GetImage("bunker").GetWidth() / 2, false);
                }
                m_RcManager->SetValue("FRIENDS_NB", m_Guns.size() - 1);
            }
            else if(m_Enemies[i].specialSkill.find("friend-box-two") != -1)
            {
                if(m_Guns.size() == 2)
                {
                    NewGun(m_RcManager->GetValue("GAME_SCREEN_WIDTH") - m_RcManager->GetImage("bunker").GetWidth() / 2, false);
                }
                else if(m_Guns.size() == 1)
                {
                    NewGun(m_RcManager->GetImage("bunker").GetWidth() / 2, false);
                }
                m_RcManager->SetValue("FRIENDS_NB", m_Guns.size() - 1);
            }

            m_Enemies.erase(m_Enemies.begin() + i);
            continue; // Important : pour que l'enemie ne soit pas supprimé deux fois !

        }
        // Bullets if ship
        if(m_Enemies[i].specialSkill.find("can-shoot") != -1 && m_Enemies[i].specialValues["SPECIAL_VAL_TIME_BEFORE_LAST_SHOT"] >= m_Enemies[i].specialValues["SPECIAL_VAL_TIME_BEFORE_NEXT_SHOT"] && m_RcManager->GetValue("LIFE") > 0)
        {
            m_RcManager->PlaySnd(m_Enemies[i].sndEnemyShot);

            m_Enemies[i].specialValues["SPECIAL_VAL_TIME_BEFORE_LAST_SHOT"] = 0;

            StructEnemy enemy;
            enemy = m_RcManager->GetEnemyFromId(floatToString(m_Enemies[i].specialValues["SPECIAL_VAL_BULLET_ID"]));
            enemy.spr.SetPosition(burstPos);
            m_Enemies.push_back(enemy);
        }
    }
}

//------------------------------------------------------------------------------
void GameScreen::DebrisMove(const float &frameTime)
{
    // Shield
    if(m_shieldAlpha > m_RcManager->GetValue("MIN_SHIELD_ALPHA"))
    {
        if(m_shieldAlpha - frameTime * 1 < m_RcManager->GetValue("MIN_SHIELD_ALPHA"))
        {
            m_shieldAlpha = m_RcManager->GetValue("MIN_SHIELD_ALPHA");
        }
        else
        {
            m_shieldAlpha -= frameTime * m_RcManager->GetValue("SHIELD_ALPHA_DECREASE_SPEED");
        }
    }

    // Debris
    for(int i = 0; i < m_Debris.size(); i++)
    {
        // Move
        m_Debris[i].spr.SetPosition(m_Debris[i].spr.GetPosition().x + m_Debris[i].speed.x * frameTime, m_Debris[i].spr.GetPosition().y + m_Debris[i].speed.y * frameTime);

        // Delete if out of the screen
        if(m_Debris[i].spr.GetPosition().x < -ITEMS_ERASE_BORDER || m_Debris[i].spr.GetPosition().x > m_RcManager->GetValue("GAME_SCREEN_WIDTH") + ITEMS_ERASE_BORDER || m_Debris[i].spr.GetPosition().y < -ITEMS_ERASE_BORDER || m_Debris[i].spr.GetPosition().y > m_RcManager->GetValue("GAME_SCREEN_HEIGHT") + ITEMS_ERASE_BORDER)
        {
            m_Debris.erase(m_Debris.begin() + i);
            continue;
        }

        // Delete if no more life time
        if(m_Debris[i].lifeTimeLeft <= 0)
        {
            m_Debris.erase(m_Debris.begin() + i);
            continue;
        }
        else
        {
            if(m_Debris[i].lifeTimeLeft - frameTime >= 0)
            {
                m_Debris[i].lifeTimeLeft -= frameTime;
            }
            else
            {
                m_Debris[i].lifeTimeLeft = 0;
            }

            char color = m_Debris[i].lifeTimeLeft * 255 / m_Debris[i].lifeTime;
            m_Debris[i].spr.SetColor(sf::Color(color,color,color, color));
        }
    }
}

//------------------------------------------------------------------------------
void GameScreen::RotateGun(const int &gunId, const float &rotation)
{
    const float min = m_RcManager->GetValue("SHOT_ANGLE_MIN"), max = m_RcManager->GetValue("SHOT_ANGLE_MAX");
    float shootAngle = m_Guns[gunId].GunSpr.GetRotation() + float(rotation);

    if(shootAngle > 180)
    {
        shootAngle -= 360;
    }

    if(shootAngle < min)
    {
        shootAngle = min;
    }
    else if(shootAngle > max)
    {
        shootAngle = max;
    }

    m_Guns[gunId].GunSpr.SetRotation(shootAngle);
}
//------------------------------------------------------------------------------
void GameScreen::AddEnemies()
{
    while(m_Enemies.size() < int(m_RcManager->GetValue("ENEMIES_NB")))
    {
        m_Enemies.push_back(m_RcManager->NewEnemy());
    }
}

//------------------------------------------------------------------------------
GameScreen::GameScreen(ConfigObj *configObj) : Screen(configObj)
{

}
//------------------------------------------------------------------------------
void GameScreen::IAComputerPlayer(const float &frameTime)
{
    const float maxDist = m_RcManager->GetValue("GAME_SCREEN_WIDTH") * m_RcManager->GetValue("GAME_SCREEN_WIDTH") + m_RcManager->GetValue("GAME_SCREEN_HEIGHT") * m_RcManager->GetValue("GAME_SCREEN_HEIGHT");

    for(int i = 0; i < m_Guns.size(); i++)
    {
        if(m_Guns[i].m_isHuman == false && m_RcManager->GetValue("LIFE") > 0)
        {
            ////////////// Plus bas
            #if 0
            int cibleId = 0;

            float plusBas = 0;
            for(int j = 0; j < m_Enemies.size(); j++)
            {
                if(m_Enemies[j].spr.GetPosition().y > plusBas && m_Enemies[j].domages > 0)
                {
                    plusBas = m_Enemies[j].spr.GetPosition().y;
                    cibleId = j;
                }
            }
            #endif

            ////////////// Plus proche du gun
            int cibleId = 0;

            float minDist = maxDist;
            for(int j = 0; j < m_Enemies.size(); j++)
            {
                float Ex = m_Guns[i].GunSpr.GetPosition().x - (m_Enemies[j].spr.GetPosition().x + m_Enemies[j].spr.GetImage()->GetWidth() / 2);
                float Ey = m_Guns[i].GunSpr.GetPosition().y - (m_Enemies[j].spr.GetPosition().y + m_Enemies[j].spr.GetImage()->GetHeight());
                float dist = sqrt(Ex * Ex + Ey * Ey);

                if(dist < minDist && m_Enemies[j].domages > 0)
                {
                    minDist = dist;
                    cibleId = j;
                }
            }

            if(m_Enemies[cibleId].domages > 0) // Si que des gentils... On ne tire pas.
            {
                sf::Vector2<float> shootPos = m_Enemies[cibleId].spr.GetPosition();

                // Soit E, enemy. Origine du repert : base du gun.
                float Ex = m_Guns[i].GunSpr.GetPosition().x - (m_Enemies[cibleId].spr.GetPosition().x + m_Enemies[cibleId].spr.GetImage()->GetWidth() / 2);
                float Ey = m_Guns[i].GunSpr.GetPosition().y - (m_Enemies[cibleId].spr.GetPosition().y + m_Enemies[cibleId].spr.GetImage()->GetHeight());

                float angle = atan(Ex/Ey) * RAD_TO_DEG;

                m_Guns[i].GunSpr.SetRotation(angle);

                if(Ey < m_RcManager->GetValue("GAME_SCREEN_HEIGHT") / 3 && m_Guns[i].m_timeBeforeLastLaser >= m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER_FRIEND")) // Si enemy bas ==> laser
                {
                    m_RcManager->PlaySnd("snd-laser");
                    m_Guns[i].m_laserBulletLeft = m_RcManager->GetValue("LASER_BULLET_NB_FRIEND");
                    m_Guns[i].m_timeBeforeLastLaser = 0;
                }
                else if(m_Guns[i].m_timeBeforeLastShot >= m_RcManager->GetValue("TIME_BEFORE_NEXT_SHOT_FRIEND"))
                {
                    Shoot(i);
                    m_Guns[i].m_timeBeforeLastShot = 0;
                }
            }
        }
    }
}
//------------------------------------------------------------------------------
int GameScreen::Run (sf::RenderWindow &App)
{
    // Params
    App.SetView(sf::View(sf::FloatRect(0, 0, m_RcManager->GetValue("GAME_SCREEN_WIDTH"), m_RcManager->GetValue("GAME_SCREEN_HEIGHT"))));
    App.ShowMouseCursor(false);

    // VerticalSync
    #if V_SYNC_IN_GAME
    App.UseVerticalSync(true);
    #else
    App.UseVerticalSync(false);
    #endif

    // Nouveau jeu
    if(m_ConfigObj->GetGameState() == GAME_REQUEST_NEW)
    {
        NewGame();
        m_ConfigObj->SetGameState(GAME_PLAY);
    }

    int previousLevel = int(m_RcManager->GetValue("LEVEL"));

    // Input, pour touches clavier.
    const sf::Input& Input = App.GetInput();

    sf::Sprite GameBackgroundSpr(m_RcManager->GetImage("game-background(1)"));

    // Wall + shield
    sf::Sprite ShieldSpr(m_RcManager->GetImage("shield"));
    ShieldSpr.SetPosition(0, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - ShieldSpr.GetImage()->GetHeight());

    sf::Sprite BottomWallSpr(m_RcManager->GetImage("wall"));
    BottomWallSpr.SetPosition(0, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - BottomWallSpr.GetImage()->GetHeight());

    // Sight
    sf::Sprite SightSpr(m_RcManager->GetImage("sight"));
    SightSpr.SetCenter(SightSpr.GetImage()->GetWidth() / 2, SightSpr.GetImage()->GetHeight());
    SightSpr.SetPosition(m_Guns[0].GunSpr.GetPosition()); // Attention : sprite center has moved !

    // Info + Progress bar
    sf::Sprite ShieldLogoSpr(m_RcManager->GetImage("shield-logo"));
    ShieldLogoSpr.SetPosition(m_RcManager->GetValue("GAME_SCREEN_WIDTH") - 60 - ShieldLogoSpr.GetImage()->GetWidth(), m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 35);

    sf::String LifeString;
    LifeString.SetPosition(m_RcManager->GetValue("GAME_SCREEN_WIDTH") - 55, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 40);
    LifeString.SetColor(sf::Color(255, 0, 0));
    LifeString.SetSize(20);

    sf::Shape NextShootProgressBar;
    sf::Shape NextLaserProgressBar;

    float timeBeforeLastFinalBurst = 0;

    // Start game loop
    while (1)
    {
        float frameTime = App.GetFrameTime() * m_RcManager->GetValue("TIME_SPEED");
        for(int i = 0; i < m_Guns.size(); i++)
        {
            m_Guns[i].m_timeBeforeLastShot += frameTime;
            m_Guns[i].m_timeBeforeLastLaser += frameTime;
            m_Guns[i].m_timeBeforeLastLaserBullet += frameTime;
        }
        for(int i = 0; i < m_Enemies.size(); i++)
        {
            if(m_Enemies[i].specialValues.find("SPECIAL_VAL_TIME_BEFORE_LAST_SHOT") != m_Enemies[i].specialValues.end())
            {
                m_Enemies[i].specialValues["SPECIAL_VAL_TIME_BEFORE_LAST_SHOT"] += frameTime;
            }
        }

        // Level up
        if(int(m_RcManager->GetValue("LEVEL")) != previousLevel) // Le score requis pour le niveau suivant augmente de 5 * niveau à chaque niveau.
        {
            m_RcManager->PlaySnd("snd-level-up");
            m_RcManager->SetValue("LIFE", m_RcManager->GetValue("LIFE") + m_RcManager->GetValue("LEVEL_UP_LIFE"));

            StructDebris debris;
            debris.lifeTime = debris.lifeTimeLeft = 4;
            debris.speed.x = 0;
            debris.speed.y = 0;
            debris.spr.SetImage(m_RcManager->GetImage("level-up"));
            debris.spr.SetPosition(m_RcManager->GetValue("GAME_SCREEN_WIDTH") / 2 - debris.spr.GetImage()->GetWidth() / 2, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") / 2 - debris.spr.GetImage()->GetHeight() / 2);
            m_Debris.push_back(debris);

            //m_RcManager->SetValue("LEVEL", int(m_RcManager->GetValue("LEVEL")) + 1);
            previousLevel = int(m_RcManager->GetValue("LEVEL"));
            if(m_RcManager->ImageExists(std::string("game-background(") + intToString(int(m_RcManager->GetValue("LEVEL"))) + std::string(")")))
            {
                GameBackgroundSpr.SetImage(m_RcManager->GetImage(std::string("game-background(") + intToString(int(m_RcManager->GetValue("LEVEL"))) + std::string(")")));
            }
        }

        AddEnemies();

        // Process events
        sf::Event Event;
        while (App.GetEvent(Event))
        {
            // Close window : exit
            if(Event.Type == sf::Event::Closed)
            {
                return -1;
            }

            // Touche 'echap' appuyée
            if(Event.Type == sf::Event::KeyPressed)
            {
                switch (Event.Key.Code)
                {
                    case sf::Key::Escape:
                        return 1;
                        break;
                }
            }
        }

        if(m_Guns[0].m_laserBulletLeft <= 0) // On peut bouger seulement si pas de laser
        {
            if(Input.IsKeyDown(sf::Key::Left) && m_RcManager->GetValue("LIFE") > 0)
            {
                RotateGun(0, GUN_ROTATION_KEYBOARD_SPEED * App.GetFrameTime());
            }

            if(Input.IsKeyDown(sf::Key::Right) && m_RcManager->GetValue("LIFE") > 0)
            {
                RotateGun(0, -GUN_ROTATION_KEYBOARD_SPEED * App.GetFrameTime());
            }

            RotateGun(0, GUN_ROTATION_MOUSE_SPEED * (float(App.GetWidth() / 2) - float(Input.GetMouseX())));
        }
        App.SetCursorPosition(App.GetWidth() / 2, App.GetHeight() / 2); // Pr Eviter de heurter les bords.

        // Bullets
        if((Input.IsKeyDown(sf::Key::Space) || Input.IsMouseButtonDown(sf::Mouse::Left)) && m_Guns[0].m_timeBeforeLastShot >= m_RcManager->GetValue("TIME_BEFORE_NEXT_SHOT") && m_RcManager->GetValue("LIFE") > 0)
        {
            Shoot(0);
            m_Guns[0].m_timeBeforeLastShot = 0;
        }

        //Laser
        if((Input.IsKeyDown(sf::Key::LControl) || Input.IsMouseButtonDown(sf::Mouse::Right)) && m_Guns[0].m_timeBeforeLastLaser >= m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER") && m_RcManager->GetValue("LIFE") > 0)
        {
            m_RcManager->PlaySnd("snd-laser");
            m_Guns[0].m_laserBulletLeft = m_RcManager->GetValue("LASER_BULLET_NB");
            m_Guns[0].m_timeBeforeLastLaser = 0;
        }

        for(int i = 0; i < m_Guns.size(); i++)
        {
            if(m_Guns[i].m_laserBulletLeft > 0 && m_Guns[i].m_timeBeforeLastLaserBullet >= m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER_BULLET") && m_RcManager->GetValue("LIFE") > 0)
            {
                Shoot(i, true);
                m_Guns[i].m_timeBeforeLastLaserBullet = 0;
                m_Guns[i].m_laserBulletLeft--;
            }
        }


        // Gestion des éléments
        if(m_RcManager->GetValue("LIFE") > 0) // Seulement si life > 0
        {
            BulletMove(frameTime);
            EnemiesMove(frameTime);
            ShieldSpr.SetColor(sf::Color(255,255,255, m_shieldAlpha));
            IAComputerPlayer(frameTime);
        }
        DebrisMove(frameTime);

        // Infos
        LifeString.SetText(intToString(int(m_RcManager->GetValue("LIFE")))); // Fn avec int pour garder la partie entiére
        if(m_RcManager->GetValue("LIFE") > 0)
        {
            // Bullets
            float barSize;
            if(m_Guns[0].m_timeBeforeLastShot >= m_RcManager->GetValue("TIME_BEFORE_NEXT_SHOT"))
            {
                barSize = m_RcManager->GetValue("PROGRESS_BAR_WIDTH");
            }
            else
            {
                barSize = m_Guns[0].m_timeBeforeLastShot * m_RcManager->GetValue("PROGRESS_BAR_WIDTH") / m_RcManager->GetValue("TIME_BEFORE_NEXT_SHOT");
            }
            NextShootProgressBar = sf::Shape::Rectangle(20, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 20, 20 + barSize, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 30, sf::Color(0,255,0));

            // Laser
            if(m_Guns[0].m_timeBeforeLastLaser >= m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER"))
            {
                barSize = m_RcManager->GetValue("PROGRESS_BAR_WIDTH");
            }
            else
            {
                barSize = m_Guns[0].m_timeBeforeLastLaser * m_RcManager->GetValue("PROGRESS_BAR_WIDTH") / m_RcManager->GetValue("TIME_BEFORE_NEXT_LASER");
            }
            NextLaserProgressBar = sf::Shape::Rectangle(20, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 40, 20 + barSize, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - 50, sf::Color(255,0,0));
        }

        // Si plus de vie
        if(m_RcManager->GetValue("LIFE") <= 0)
        {
            m_RcManager->SetValue("LIFE", 0);


            BottomWallSpr.SetColor(sf::Color(m_RcManager->GetValue("LOST_BUILDING_GRAY_LEVEL"), m_RcManager->GetValue("LOST_BUILDING_GRAY_LEVEL"), m_RcManager->GetValue("LOST_BUILDING_GRAY_LEVEL")));
            for(int i = 0; i < m_Guns.size(); i++)
            {
                m_Guns[i].BunkerSpr.SetColor(sf::Color(m_RcManager->GetValue("LOST_BUILDING_GRAY_LEVEL"), m_RcManager->GetValue("LOST_BUILDING_GRAY_LEVEL"), m_RcManager->GetValue("LOST_BUILDING_GRAY_LEVEL")));
            }
            GameBackgroundSpr.SetColor(sf::Color(m_RcManager->GetValue("LOST_BACKGROUND_GRAY_LEVEL"), m_RcManager->GetValue("LOST_BACKGROUND_GRAY_LEVEL"), m_RcManager->GetValue("LOST_BACKGROUND_GRAY_LEVEL")));

            if(timeBeforeLastFinalBurst > m_RcManager->GetValue("FINAL_BURST_FREQ"))
            {
                m_RcManager->PlaySnd("snd-lost-burst");

                StructBurstInfo tmpStruct = {sf::Vector2f(0,0), -m_RcManager->GetValue("FINAL_BURST_DEBRIS_ANGLE") / 2, m_RcManager->GetValue("FINAL_BURST_DEBRIS_ANGLE") / 2, false, false, false};
                tmpStruct.pos = sf::Vector2<float>(sf::Randomizer::Random(0.f, m_RcManager->GetValue("GAME_SCREEN_WIDTH")), sf::Randomizer::Random(float(m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - BottomWallSpr.GetImage()->GetHeight()), float(m_RcManager->GetValue("GAME_SCREEN_HEIGHT"))));
                CreateExplosion(tmpStruct);

                tmpStruct.pos = sf::Vector2<float>(sf::Randomizer::Random(0.f, m_RcManager->GetValue("GAME_SCREEN_WIDTH")), sf::Randomizer::Random(float(m_RcManager->GetValue("GAME_SCREEN_HEIGHT") - BottomWallSpr.GetImage()->GetHeight()), float(m_RcManager->GetValue("GAME_SCREEN_HEIGHT"))));
                CreateExplosion(tmpStruct);

                timeBeforeLastFinalBurst = 0;
            }
            else
            {
                timeBeforeLastFinalBurst += frameTime;
            }

            // Score
            if(!m_scoreSent) //Si on n'a pas dèjà donné le score...
            {
                m_haveScore = m_ConfigObj->AddScore(m_RcManager->GetValue("SCORE"));
                m_scoreSent = true;
            }

            if(m_ConfigObj->GetCurrentModeDirName() == "Test") // Si mode Test ==> On rejou en boucle
            {
                NewGame();
            }
        }


        // Affichage
        App.Clear();

        App.Draw(GameBackgroundSpr);

        if(m_RcManager->GetValue("LIFE") > 0) // Seulement si life > 0
        {
            for(int i = 0; i < m_BulletsList.size(); i++)
            {
                App.Draw(m_BulletsList[i].spr);
            }
            for(int i = 0; i < m_Enemies.size(); i++)
            {
                App.Draw(m_Enemies[i].spr);
            }

            for(int i = 0; i < m_Guns.size(); i++)
            {
                if(m_Guns[i].isSight)
                {
                    SightSpr.SetPosition(m_Guns[i].GunSpr.GetPosition()); // Attention : sprite center has moved !
                    SightSpr.SetRotation(m_Guns[i].GunSpr.GetRotation()); // Attention : sprite center has moved !
                    App.Draw(SightSpr);
                }
            }
        }

        App.Draw(BottomWallSpr);

        for(int i = 0; i < m_Guns.size(); i++)
        {
            if(m_RcManager->GetValue("LIFE") > 0)
            {
                App.Draw(m_Guns[i].GunSpr);
            }
            App.Draw(m_Guns[i].BunkerSpr);
        }

        for(int i = 0; i < m_Debris.size(); i++)
        {
            App.Draw(m_Debris[i].spr);
        }

        if(m_RcManager->GetValue("LIFE") > 0) // Bouclier seulement si life > 0
        {
            App.Draw(ShieldSpr);
            App.Draw(NextShootProgressBar);
            App.Draw(NextLaserProgressBar);
        }
        else
        {
            std::string tempStr = "Perdu !";
            tempStr += std::string("\n\n") + m_ConfigObj->GetScores();
            if(m_haveScore)
            {
                tempStr += std::string("\nVous êtes dans le top ") + intToString(NB_SCORE_SAVED) + std::string(" !");
            }
            sf::String deadMsg(tempStr);
            deadMsg.SetPosition(m_RcManager->GetValue("GAME_SCREEN_WIDTH") / 2 - deadMsg.GetRect().GetWidth() / 2, m_RcManager->GetValue("GAME_SCREEN_HEIGHT") / 2 - deadMsg.GetRect().GetHeight() / 2);
            App.Draw(deadMsg);
        }

        sf::String Score(std::string("Score : ") + intToString(m_RcManager->GetValue("SCORE")));
        App.Draw(Score);

        Score.SetText(std::string("Niveau : ") + intToString(int(m_RcManager->GetValue("LEVEL"))));
        Score.SetPosition(0.f, 30.f);
        App.Draw(Score);

        if(m_ConfigObj->IsFpsOn())
        {
            Score.SetText(std::string("FPS : ") + floatToString(1/App.GetFrameTime()));
            Score.SetPosition(0.f, 60.f);
            App.Draw(Score);
        }

        App.Draw(ShieldLogoSpr);
        App.Draw(LifeString);

        // Display window contents on screen
        App.Display();
    }

    return -1;
}
