///File : MenuScreen.cpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MenuScreen.hpp"
//------------------------------------------------------------------------------

void LoadModePresetation(void* UserData)
{
    StructModePresentation &modePresentation = *static_cast<StructModePresentation*>(UserData);
    modePresentation.isOk = false;

    std::string imgFilename;


    std::string filename = modePresentation.modeDir + std::string("/mode-presentation.conf");

    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture

    if(file)  // si l'ouverture a fonctionné
    {
        std::string str;

        std::getline(file, modePresentation.modeName);
        std::getline(file, modePresentation.auther);
        std::getline(file, imgFilename);

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't load config \"" << filename << "\" !";
        modePresentation.modeName = "Unknown";
        modePresentation.auther = "Unknown";
        imgFilename = "no";
    }

    if(!imgFilename.empty() && imgFilename != "no")
    {
        imgFilename = modePresentation.modeDir + std::string("/") + imgFilename;
        if(!modePresentation.screenShot.LoadFromFile(imgFilename))
        {
            ERROR_FLUX << "Can't load image \"" << imgFilename << "\" ! Will now exit.";
            exit(-1);
        }
    }
    else
    {
        modePresentation.screenShot = sf::Image(); // Img vide.
    }

    modePresentation.isOk = true;
}
//------------------------------------------------------------------------------

MenuScreen::MenuScreen(ConfigObj *configObj) : Screen(configObj)
{
    m_menuChoice = 0;
    m_menuId = MAIN_MENU;
    m_modePresentationThread = new sf::Thread(&LoadModePresetation, &m_modePresentation);
}
//------------------------------------------------------------------------------
MenuScreen::~MenuScreen()
{
    delete m_modePresentationThread;
}
//------------------------------------------------------------------------------
void MenuScreen::MenuUp()
{
    m_RcManager->PlaySnd("snd-menu-change");
    if(m_menuChoice == 0)
    {
        m_menuChoice = m_Menu.size() -1;
    }
    else
    {
        m_menuChoice--;
    }
}
//------------------------------------------------------------------------------
void MenuScreen::MenuDown()
{
    m_RcManager->PlaySnd("snd-menu-change");
    if(m_menuChoice == m_Menu.size() -1)
    {
        m_menuChoice = 0;
    }
    else
    {
        m_menuChoice++;
    }
}
//------------------------------------------------------------------------------
bool MenuScreen::MenuAction(int &returnValue)
{
    m_RcManager->PlaySnd("snd-menu-action");

    returnValue = 0;
    if(m_menuId == MAIN_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                if(m_ConfigObj->GetGameState() == GAME_REQUEST_NEW)
                {
                    returnValue = 2;
                    return true;
                }
                else
                {
                    m_menuId = PLAY_MENU;
                    CreateMenu();
                    return false;
                }
                break;
            case 1:
                m_menuId = OPTION_MENU;
                CreateMenu();
                return false;
                break;
            case 2:
                returnValue = 3;
                return true;
                break;
            case 3:
                m_menuId = QUIT_MENU;
                CreateMenu();
                return false;
                break;
        }
    }
    else if(m_menuId == OPTION_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                m_menuId = MAIN_MENU;
                CreateMenu();
                m_menuChoice = 1;
                return false;
                break;
            case 1:
                m_menuId = SOUND_MENU;
                CreateMenu();
                return false;
                break;
            case 2:
                m_menuId = VIDEO_MENU;
                CreateMenu();
                return false;
                break;
            case 3:
                m_tmpMode = m_ConfigObj->GetCurrentModeDirName();
                m_menuId = MODE_MENU;
                CreateMenu();

                // On carge la presentation
                m_modePresentation.modeDir = m_ConfigObj->GetTempModeTotalDir(m_ConfigObj->GetCurrentModeDirName());
                m_modePresentationThread->Launch();

                m_menuChoice = 1;
                return false;
                break;
        }
    }
    else if(m_menuId == VIDEO_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                m_menuId = OPTION_MENU;
                CreateMenu();
                m_menuChoice = 2;
                return false;
                break;
            case 1:
                m_ConfigObj->SetFullScreenOn(!m_ConfigObj->IsFullScreenOn());
                CreateMenu();
                m_menuChoice = 1;
                return false;
                break;
            case 2:
                m_ConfigObj->ChangeVideoMode();
                CreateMenu();
                m_menuChoice = 2;
                return false;
                break;
            case 3:
                m_ConfigObj->SetFpsOn(!m_ConfigObj->IsFpsOn());
                CreateMenu();
                m_menuChoice = 3;
                return false;
                break;
        }
    }
    else if(m_menuId == QUIT_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                m_menuId = MAIN_MENU;
                CreateMenu();
                return false;
                break;
            case 1:
                returnValue = -1;
                return true;
                break;
        }
    }
    else if(m_menuId == PLAY_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                m_menuId = MAIN_MENU;
                returnValue = 2;
                return true;
                break;
            case 1:
                m_ConfigObj->SetGameState(GAME_REQUEST_NEW);
                m_menuId = MAIN_MENU;
                returnValue = 2;
                return true;
                break;
            case 2:
                m_menuId = MAIN_MENU;
                CreateMenu();
                return false;
                break;
        }
    }
    else if(m_menuId == MODE_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                m_menuId = OPTION_MENU;
                CreateMenu();
                m_menuChoice = 3;
                return false;
                break;
            case 1:
                m_tmpMode = m_ConfigObj->GetNextMode(m_tmpMode);

                m_modePresentation.modeDir = m_ConfigObj->GetTempModeTotalDir(m_tmpMode);
                m_modePresentationThread->Launch();

                CreateMenu();
                m_menuChoice = 1;
                return false;
                break;
            case 2:
                // Si on chanhe de mode
                if(m_tmpMode != m_ConfigObj->GetCurrentModeDirName())
                {
                    m_ConfigObj->SetCurrentModeDirName(m_tmpMode);
                    m_ConfigObj->SaveAll();
                    m_menuId = MAIN_MENU;
                    returnValue = 4;
                    return true;
                }
                else
                {
                    m_menuId = OPTION_MENU;
                    CreateMenu();
                    m_menuChoice = 3;
                    return false;
                }
                break;
        }
    }
    else if(m_menuId == SOUND_MENU)
    {
        switch(m_menuChoice)
        {
            case 0:
                m_menuId = OPTION_MENU;
                CreateMenu();
                m_menuChoice = 1;
                return false;
                break;
            case 1:
                m_RcManager->SetMusicOn(!m_RcManager->IsMusicOn());
                CreateMenu();
                m_menuChoice = 1;
                return false;
                break;
            case 2:
                m_RcManager->SetSoundOn(!m_RcManager->IsSoundOn());
                CreateMenu();
                m_menuChoice = 2;
                return false;
                break;
        }
    }
}
//------------------------------------------------------------------------------
void MenuScreen::CreateMenu()
{
    // Menu content
    m_Menu.clear();

    m_menuChoice = 0;
    std::string str;

    switch (m_menuId)
    {
    case MAIN_MENU:
        m_MenuTitle.SetText("Menu principal");
        m_Menu.push_back(sf::String("Jouer"));
        m_Menu.push_back(sf::String("Options"));
        m_Menu.push_back(sf::String("A propos"));
        m_Menu.push_back(sf::String("Quitter"));
        break;
    case QUIT_MENU:
        m_MenuTitle.SetText("Quitter ?");
        m_Menu.push_back(sf::String("Non"));
        m_Menu.push_back(sf::String("Oui"));
        break;
    case OPTION_MENU:
        m_MenuTitle.SetText("Options");
        m_Menu.push_back(sf::String("Retour"));

        m_Menu.push_back(sf::String("Son"));

        m_Menu.push_back(sf::String("Video"));

        if(m_ConfigObj->GetModeDirNamesList().size() > 1)
        {
            m_Menu.push_back(sf::String("Modes"));
        }
        break;
    case PLAY_MENU:
        m_MenuTitle.SetText("Jouer ?");
        m_Menu.push_back(sf::String("Continuer"));
        m_Menu.push_back(sf::String("Nouveau"));
        m_Menu.push_back(sf::String("Retour"));
        break;
    case MODE_MENU:
        m_MenuTitle.SetText("Modes");
        m_Menu.push_back(sf::String("Retour"));

        str = "Mode : ";
        str += m_tmpMode;
        m_Menu.push_back(sf::String(str));

        m_Menu.push_back(sf::String("Ok"));
        break;
    case SOUND_MENU:
        m_MenuTitle.SetText("Son");
        m_Menu.push_back(sf::String("Retour"));

        str = "Musique : ";
        str += m_RcManager->IsMusicOn() ? std::string("Oui") : std::string("Non");
        m_Menu.push_back(sf::String(str));

        str = "Son : ";
        str += m_RcManager->IsSoundOn() ? std::string("Oui") : std::string("Non");
        m_Menu.push_back(sf::String(str));
        break;
    case VIDEO_MENU:
        m_MenuTitle.SetText("Video");
        m_Menu.push_back(sf::String("Retour"));

        str = "Plein Ecran : ";
        str += m_ConfigObj->IsFullScreenOn() ? std::string("Oui") : std::string("Non");
        m_Menu.push_back(sf::String(str));

        str = "Mode : ";
        str += m_ConfigObj->GetCurrentVideoModeText();
        m_Menu.push_back(sf::String(str));

        str = "FPS : ";
        str += m_ConfigObj->IsFpsOn() ? std::string("Oui") : std::string("Non");
        m_Menu.push_back(sf::String(str));
        break;
    }

    // Menu pos
    const float FIRST_MENU_X_POS = 600;
    const float FIRST_MENU_Y_POS = 400;
    const float MENU_INTER_SIZE_X = 30;
    const float MENU_INTER_SIZE_Y = 150;

    m_MenuTitle.SetPosition(550, 50);
    m_MenuTitle.SetFont(m_RcManager->GetFont("menu-font"));
    m_MenuTitle.SetSize(80);
    m_MenuTitle.SetColor(sf::Color(255, 60, 60));

    m_Menu[0].SetPosition(FIRST_MENU_X_POS, FIRST_MENU_Y_POS);
    for(int i = 1; i < m_Menu.size(); i++)
    {
        m_Menu[i].SetPosition(m_Menu[i - 1].GetPosition().x + MENU_INTER_SIZE_X, m_Menu[i - 1].GetPosition().y + MENU_INTER_SIZE_Y);
    }

    for(int i = 0; i < m_Menu.size(); i++)
    {
        m_Menu[i].SetFont(m_RcManager->GetFont("menu-font"));
        m_Menu[i].SetSize(72);
    }
}
//------------------------------------------------------------------------------
int MenuScreen::Run (sf::RenderWindow &App)
{
    // Params
    App.SetView(sf::View(sf::FloatRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)));
    App.ShowMouseCursor(false);

    // VerticalSync
    #if V_SYNC_IN_MENU
    App.UseVerticalSync(true);
    #else
    App.UseVerticalSync(false);
    #endif

    CreateMenu();

    sf::Sprite MenuBackgroundSpr(m_RcManager->GetImage("menu-background"));

    sf::Sprite ArrowSpr(m_RcManager->GetImage("menu-cursor"));
    ArrowSpr.SetX(App.GetInput().GetMouseX() + -5);
    ArrowSpr.SetY(App.GetInput().GetMouseY());

    sf::Sprite modeScreenShotSpr;
    sf::String tmpSfString;
    sf::Shape modePresentationBackground;

    sf::Shape menuSelectRect;

    int returnValue = -1;
    while (1)
    {
        sf::Event Event;
        while (App.GetEvent(Event))
        {
            // Window closed
            if (Event.Type == sf::Event::Closed)
            {
                return (-1);
            }
            //Key pressed
            if (Event.Type == sf::Event::KeyPressed)
            {
                switch (Event.Key.Code)
                {
                    case sf::Key::Up:
                        MenuUp();
                        break;
                    case sf::Key::Down:
                        MenuDown();
                        break;
                    case sf::Key::Return:
                        if(MenuAction(returnValue))
                        {
                            return returnValue;
                        }
                        break;
                    case sf::Key::Escape:
                        m_menuChoice = 0; // Choix retour
                        if(MenuAction(returnValue))
                        {
                            return returnValue;
                        }
                        break;
                    default:
                        break;
                }
            }
            // Mouse Wheel
            if (Event.Type == sf::Event::MouseWheelMoved)
            {
                if(Event.MouseWheel.Delta > 0)
                {
                    MenuUp();
                }
                else
                {
                    MenuDown();
                }
            }
            // Mouse move
            if (Event.Type == sf::Event::MouseMoved)
            {
                // On multiplie la positionde la souris par (SCREEN_WIDTH / App.GetWidth()) car, si la résolution est trop petite, celle ci ne va pas jusqu'au bout de l'écran.
                ArrowSpr.SetX((SCREEN_WIDTH / App.GetWidth()) * Event.MouseMove.X - 5);
                ArrowSpr.SetY((SCREEN_HEIGHT / App.GetHeight()) * Event.MouseMove.Y);
                for(int i = 0; i < m_Menu.size(); i++)
                {
                    if(m_Menu[i].GetRect().Contains((SCREEN_WIDTH / App.GetWidth()) * Event.MouseMove.X, (SCREEN_HEIGHT / App.GetHeight()) * Event.MouseMove.Y) && m_menuChoice != i) // Si mouse in && menu change (pour pas rejouer le son)
                    {
                        m_RcManager->PlaySnd("snd-menu-change");
                        m_menuChoice = i;
                    }
                }
            }
            // Mouse click
            if (Event.Type == sf::Event::MouseButtonReleased)
            {
                for(int i = 0; i < m_Menu.size(); i++)
                {
                    if(m_Menu[i].GetRect().Contains((SCREEN_WIDTH / App.GetWidth()) * App.GetInput().GetMouseX(), (SCREEN_HEIGHT / App.GetHeight()) * App.GetInput().GetMouseY()))
                    {
                        m_menuChoice = i;
                        if(MenuAction(returnValue))
                        {
                            return returnValue;
                        }
                    }
                }
            }
        }

        //Drawing
        App.Clear();

        App.Draw(MenuBackgroundSpr);

        for(int i = 0; i < m_Menu.size(); i++)
        {
            if(m_menuChoice == i)
            {
                m_Menu[i].SetColor(sf::Color(255, 240, 240));
                menuSelectRect = sf::Shape::Rectangle(m_Menu[i].GetRect().Left - 15, m_Menu[i].GetRect().Top, m_Menu[i].GetRect().Right + 15, m_Menu[i].GetRect().Bottom + 15, sf::Color(0,0,0));
                App.Draw(menuSelectRect);
            }
            else
            {
                m_Menu[i].SetColor(sf::Color(255, 60, 60));
            }
            App.Draw(m_Menu[i]);
        }
        App.Draw(m_MenuTitle);

        if(m_menuId == MODE_MENU && m_modePresentation.isOk)
        {
            modePresentationBackground = sf::Shape::Rectangle(48, 418, 52 + m_modePresentation.screenShot.GetWidth(), 502 + m_modePresentation.screenShot.GetHeight(), sf::Color(0,0,0));
            App.Draw(modePresentationBackground);

            modeScreenShotSpr.SetImage(m_modePresentation.screenShot);
            modeScreenShotSpr.SetPosition(50, 500);
            App.Draw(modeScreenShotSpr);

            tmpSfString.SetText(m_modePresentation.modeName);
            tmpSfString.SetPosition(50, 420);
            App.Draw(tmpSfString);


            tmpSfString.SetText(m_modePresentation.auther);
            tmpSfString.SetPosition(50, 460);
            App.Draw(tmpSfString);
        }

        App.Draw(ArrowSpr);

        App.Display();
    }

    //Never reaching this point normally, but just in case, exit the application
    return (-1);
}

//------------------------------------------------------------------------------
/// About Screen
//------------------------------------------------------------------------------

AboutScreen::AboutScreen(ConfigObj *configObj) : Screen(configObj)
{

}
//------------------------------------------------------------------------------

int AboutScreen::Run (sf::RenderWindow &App)
{
    // Params
    App.SetView(sf::View(sf::FloatRect(0, 0, 1280, 1024)));
    App.ShowMouseCursor(false);

    // VerticalSync
    #if V_SYNC_IN_MENU
    App.UseVerticalSync(true);
    #else
    App.UseVerticalSync(false);
    #endif

    // Create the sprite
    sf::Sprite AboutSpr(m_RcManager->GetImage("about"));

    // Start loop
    while (1)
    {
        // Process events
        sf::Event Event;
        while (App.GetEvent(Event))
        {
            // Close window : exit
            if(Event.Type == sf::Event::Closed)
            {
                return -1;
            }

            // Touche 'echap' appuyée
            if(Event.Type == sf::Event::KeyPressed || Event.Type == sf::Event::MouseButtonReleased)
            {
                m_RcManager->PlaySnd("snd-menu-action");
                return 1;
            }
        }

        // Clear screen
        App.Clear();

        // Display sprite in our window
        App.Draw(AboutSpr);

        // Display window contents on screen
        App.Display();
    }

    return -1;
}
