///File : RcManager.cpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "RcManager.hpp"
//------------------------------------------------------------------------------

void GetFileListFromNormalFile(StructAllInfo &m_i, const std::string &filename)
{
    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture
    if(file) // si l'ouverture a fonctionné
    {
        std::string str;
        StructFileList tempFileList;

        while(!file.eof())
        {
            std::getline(file, str);
            if(!str.empty() && str[0] != '#')
            {
                str = str.substr(0, str.find('#')); // On enléve les commentaires
                tempFileList.alias = str.substr(0, str.find('='));
                tempFileList.file = str.substr(str.find('=') + 1);
                m_i.m_FilesList.push_back(tempFileList);
            }
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't load mode files list base (\"" << filename << "\" !" << std::endl;
        exit(-1);
    }
}
//------------------------------------------------------------------------------
void GetInfo(StructAllInfo &m_i, const std::string &filename)
{
    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture
    if(file) // si l'ouverture a fonctionné
    {
        std::string str;
        StructFileList tempFileList;

        while(!file.eof())
        {
            std::getline(file, str);
            if(!str.empty() && str[0] != '#')
            {
                str = str.substr(0, str.find('#')); // On enléve les commentaires
                m_i.m_ValueMap[str.substr(0, str.find('='))] = str.substr(str.find('=') + 1);
                INFO_FLUX << str.substr(0, str.find('=')) << "=" << str.substr(str.find('=') + 1) << std::endl;
            }
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't load info file (\"" << filename << "\" !" << std::endl;
        exit(-1);
    }
}
//------------------------------------------------------------------------------
StructEnemy RcManager::CreateStructEnemy(const int &enemyId)
{
    // Etape 1 : on crée un StockEnemy avec des infos traité :
    const std::string specialWord = "SPECIAL";
    std::map<std::string, std::string> tmpStockEnemy;

    std::map<std::string, std::string>::reverse_iterator rit;
    for ( rit = m_i.m_EnemyStockList[enemyId].map.rbegin() ; rit != m_i.m_EnemyStockList[enemyId].map.rend(); rit++ )
    {
        tmpStockEnemy[rit->first] = ParseStringInfo(m_i.m_EnemyStockList[enemyId].map[rit->first]);
    }

    StructEnemy tmpEnemy;
    tmpEnemy.spr.SetImage(GetImage(tmpStockEnemy["IMAGE"]));
    tmpEnemy.spr.SetPosition(sf::Vector2f(sf::Randomizer::Random(stringToFloat(tmpStockEnemy["POS_X_MIN"]), stringToFloat(tmpStockEnemy["POS_X_MAX"])), sf::Randomizer::Random(stringToFloat(tmpStockEnemy["POS_Y_MIN"]), stringToFloat(tmpStockEnemy["POS_Y_MAX"]))));
    tmpEnemy.imageResistAlias = tmpStockEnemy["IMAGE_RESIST"];

    tmpEnemy.sndShieldShock = tmpStockEnemy["SND_SHIELD_SHOCK"];
    tmpEnemy.sndBurst = tmpStockEnemy["SND_ENEMY_BURST"];
    tmpEnemy.sndResist = tmpStockEnemy["SND_ENEMY_RESIST"];
    tmpEnemy.sndEnemyShot = tmpStockEnemy["SND_ENEMY_SHOT"];

    tmpEnemy.imageResistPos = sf::Vector2f(stringToFloat(tmpStockEnemy["IMAGE_RESIST_POS_Y"]), stringToFloat(tmpStockEnemy["IMAGE_RESIST_POS_Y"]));
    tmpEnemy.speed = sf::Vector2f(sf::Randomizer::Random(stringToFloat(tmpStockEnemy["SPEED_X_MIN"]), stringToFloat(tmpStockEnemy["SPEED_X_MAX"])), sf::Randomizer::Random(stringToFloat(tmpStockEnemy["SPEED_Y_MIN"]), stringToFloat(tmpStockEnemy["SPEED_Y_MAX"])));
    tmpEnemy.life = stringToFloat(tmpStockEnemy["LIFE"]);
    tmpEnemy.domages = stringToFloat(tmpStockEnemy["DOMAGES"]);
    tmpEnemy.specialSkill = tmpStockEnemy["SPECIAL_SKILL"];

    if(tmpStockEnemy.find("SPECIAL_VAL_TIME_BEFORE_NEXT_SHOT") != tmpStockEnemy.end())
    {
        tmpStockEnemy["SPECIAL_VAL_TIME_BEFORE_LAST_SHOT"] = 0.f;
    }

    for( rit = tmpStockEnemy.rbegin() ; rit != tmpStockEnemy.rend(); rit++ )
    {
        if(rit->first.substr(0, specialWord.size()) == specialWord)
        {
            tmpEnemy.specialValues[rit->first] = stringToFloat(tmpStockEnemy[rit->first]);
        }
    }

    return tmpEnemy;
}
//------------------------------------------------------------------------------
StructEnemy RcManager::GetEnemyFromId(const std::string &id)
{
    for(int i = 0; i < m_i.m_EnemyStockList.size(); i++)
    {
        if(m_i.m_EnemyStockList[i].map["ID"] == id)
        {
            return CreateStructEnemy(i);
        }
    }

    ERROR_FLUX << "No enemy has id \"" << id << "\" !\nWill now exit." << std::endl;
    exit (-1);
}

//------------------------------------------------------------------------------
bool RcManager::CanUseEnemy(const int &enemyId)
{
    for(int i = 0; i < m_i.m_EnemyStockList[enemyId].needs.size(); i++)
    {
        const std::string value1 = GetValueStr(m_i.m_EnemyStockList[enemyId].needs[i].varName);
        const std::string value2 = ParseStringInfo(m_i.m_EnemyStockList[enemyId].needs[i].value);

        switch(m_i.m_EnemyStockList[enemyId].needs[i].compareType)
        {
            case LOWER:
                if(!(stringToFloat(value1) < stringToFloat(value2)))
                    return false;
                break;
            case LOWER_EQUAL:
                if(!(stringToFloat(value1) <= stringToFloat(value2)))
                    return false;
                break;
            case EQUAL:
                if(!(value1 == value2)) // Pas en float pour que le "mappeur" puisse utiliser des chaines.
                    return false;
                break;
            case UPPER_EQUAL:
                if(!(stringToFloat(value1) >= stringToFloat(value2)))
                    return false;
                break;
            case UPPER:
                if(!(stringToFloat(value1) > stringToFloat(value2)))
                    return false;
                break;
            case DIFFERENT:
                if(!(value1 != value2)) // Pas en float pour que le "mappeur" puisse utiliser des chaines.
                    return false;
                break;
            default:
                ERROR_FLUX << "Unknown comparaison type !\nWill now exit." << std::endl;
                exit(-1);
                break;
        }
    }

    return true;
}

//------------------------------------------------------------------------------
StructEnemy RcManager::NewEnemy()
{
    float level = GetValue("LEVEL");

    // Etape 1 : conter sur combien de chance ça se joue...
    float chanceNb = 0;
    for(int i = 0; i < m_i.m_EnemyStockList.size(); i++)
    {
        if(CanUseEnemy(i))
        {
            chanceNb += stringToFloat(ParseStringInfo(m_i.m_EnemyStockList[i].map["CHANCE"]));
        }
    }

    // Etape 2 : On "choisi"
    int enemyId = 0;
    float chance = sf::Randomizer::Random(0.f, chanceNb);
    float currentChance = 0;
    for(int i = 0; i < m_i.m_EnemyStockList.size(); i++)
    {
        if(CanUseEnemy(i)) // Si il fait parti des prenables
        {
            currentChance += stringToFloat(ParseStringInfo(m_i.m_EnemyStockList[i].map["CHANCE"]));
            if(chance <= currentChance) // Si l'enemi en cours (i) est choisi
            {
                enemyId = i;
                break;
            }
        }
    }

    // Etape 3 : On crée un enemi tous près.
    return CreateStructEnemy(enemyId);
}
//------------------------------------------------------------------------------
void GetEnemies(StructAllInfo &m_i, const std::string &filename)
{
    const int signesTblsize = 6;
    std::string signesTbl[signesTblsize] = {"<=", "==", ">=", "!=", "<", ">"}; // /!\ : signes à 1 caractére en premier, si non, si sont trouvés à la place d'un autre.
    const int signesIntTbl[signesTblsize] = {LOWER_EQUAL, EQUAL, UPPER_EQUAL, DIFFERENT, LOWER, UPPER}; // /!\ : signes à 1 caractére en premier, si non, si sont trouvés à la place d'un autre.

    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture
    if(file) // si l'ouverture a fonctionné
    {
        std::string str, id, value;
        int signeId = -1;

        while(!file.eof())
        {
            std::getline(file, str);
            if(str == std::string("enemy:"))
            {
                std::vector<StructNeeds> enemyNeeds;

                std::map<std::string, std::string> tempEnemy;
                tempEnemy["ID"]="noId";
                tempEnemy["CHANCE"]="1";
                tempEnemy["IMAGE"]="No image set";
                tempEnemy["IMAGE_RESIST"]="no";
                tempEnemy["IMAGE_RESIST_POS_X"]="0";
                tempEnemy["IMAGE_RESIST_POS_Y"]="0";
                tempEnemy["SND_SHIELD_SHOCK"]="no";
                tempEnemy["SND_ENEMY_SHOT"]="no";
                tempEnemy["SND_BURST"]="no";
                tempEnemy["SND_RESIST"]="no";
                tempEnemy["DOMAGES"]="10";
                tempEnemy["LIFE"]="1";
                tempEnemy["SPEED_X_NIN"]="0";
                tempEnemy["SPEED_Y_MIN"]="0";
                tempEnemy["SPEED_X_MAX"]="0";
                tempEnemy["SPEED_Y_MAX"]="0";
                tempEnemy["POS_X_MIN"]="0";
                tempEnemy["POS_X_MAX"]="0";
                tempEnemy["POS_Y_MIN"]="0";
                tempEnemy["POS_Y_MAX"]="0";
                tempEnemy["SPECIAL_SKILL"]="no";
                tempEnemy["SPECIAL_VALUE"]="0";

                INFO_FLUX << "New enemy" << std::endl;
                while(!file.eof() && str != std::string("}"))
                {
                    std::getline(file, str);
                    if(!str.empty() && str[0] != '#')
                    {
                        str = str.substr(0, str.find('#')); // On enléve les commentaires

                        signeId = -1;
                        for(int i = 0; i < signesTblsize; i++)
                        {
                            if(str.find(signesTbl[i]) != std::string::npos)
                            {
                                signeId = i;
                                break;
                            }
                        }

                        if(signeId != -1)
                        {
                            StructNeeds tmpEnemyNeed;
                            tmpEnemyNeed.varName = str.substr(0, str.find(signesTbl[signeId]));
                            tmpEnemyNeed.compareType = signesIntTbl[signeId];
                            tmpEnemyNeed.value =  str.substr(str.find(signesTbl[signeId]) + signesTbl[signeId].size());

                            enemyNeeds.push_back(tmpEnemyNeed);
                        }
                        else if(str.find('=') != -1)
                        {
                            id = str.substr(0, str.find('='));
                            value = str.substr(str.find('=') + 1);

                            // Add to enemy info DB
                            INFO_FLUX << "\t" << id << "=" << value << std::endl;
                            tempEnemy[id] = value;

                            // Add to load list files that have to be loaded
                            if(id == std::string("IMAGE") || id == std::string("IMAGE_RESIST") || id == std::string("SND_SHIELD_SHOCK") || id == std::string("SND_ENEMY_SHOT") || id == std::string("SND_ENEMY_BURST") || id == std::string("SND_ENEMY_RESIST"))
                            {
                                StructFileList tmpFileList;
                                tmpFileList.file = value;
                                tmpFileList.alias = value;
                                m_i.m_FilesList.push_back(tmpFileList);
                                INFO_FLUX << "\t \\--> To load : " << id << "=" << value << std::endl;
                            }
                        }
                    }
                }
                StructStockEnemy tmpStockEnemy;
                tmpStockEnemy.map = tempEnemy;
                tmpStockEnemy.needs = enemyNeeds;

                m_i.m_EnemyStockList.push_back(tmpStockEnemy);
                INFO_FLUX << std::endl;
            }
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't load mode files list base (\"" << filename << "\" !" << std::endl;
        exit(-1);
    }
}
//------------------------------------------------------------------------------
void RcManager::SetValue(const std::string &id,const float &value)
{
    m_i.m_ValueMap[id] = floatToString(value);
}
//------------------------------------------------------------------------------
void RcManager::SetValue(const std::string &id,const std::string &value)
{
    m_i.m_ValueMap[id] = value;
}
//------------------------------------------------------------------------------
bool RcManager::ValueExists(const std::string &id)
{
    if(m_i.m_ValueMap.find(id) != m_i.m_ValueMap.end())
    {
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
std::string RcManager::Calcl(std::string value)
{
    int varBeginPos = 0;
    int varEndPos = 0;
    float tempNb = 0;

    const int tblSize = 4;
    char signs[] = "+-*/";
    char currentSign = '\0';
    int currentSignPos = -1;

    while(1)
    {
        // Bracket.
        while(value.find('(') != std::string::npos)
        {
            int bracketLevel = 1;
            int searchBeginPos = value.find('(');
            int searchPos = value.find('(');

            // On cherche la fin de la parenthése.
            while(searchPos < value.size() && bracketLevel != 0)
            {
                searchPos++;
                if(value[searchPos] == '(')
                {
                    bracketLevel++;
                }
                else if(value[searchPos] == ')')
                {
                    bracketLevel--;
                }
            }

            // On calcule le contenu
            std::string result = Calcl(value.substr(searchBeginPos + 1, searchPos - searchBeginPos - 1));

            // Cas spécial : les racines carrées
            const std::string sqrtStr("Sqrt");
            if(searchBeginPos >= sqrtStr.size())
            {
                if(value.substr(searchBeginPos - sqrtStr.size(), sqrtStr.size()) == sqrtStr)
                {
                    searchBeginPos -= sqrtStr.size(); // La pos de debut de recherche diminue de 1
                    result = floatToString(sqrt(stringToFloat(result)));
                }
            }

            const std::string intStr("int");
            if(searchBeginPos >= intStr.size())
            {
                if(value.substr(searchBeginPos - intStr.size(), intStr.size()) == intStr)
                {
                    searchBeginPos -= intStr.size(); // La pos de debut de recherche diminue de 1
                    result = floatToString(int(stringToFloat(result)));
                }
            }

            // On supprime la bracket
            value = value.erase(searchBeginPos, searchPos - searchBeginPos + 1);

            // On ajoute le résultat
            value.insert(searchBeginPos, result);
        }

        currentSign = '\0';
        currentSignPos = -1;
        for(int i = 0; i < tblSize; i++)
        {
            if(value.find(signs[i]) != std::string::npos)
            {
                if(currentSignPos == -1 || currentSignPos > value.find(signs[i], 1)) // Si premier signe trouvé ou si ce signe est plus proche du début que celui d'avant : il faut le prendre.
                {
                    currentSignPos = value.find(signs[i], 1);
                    currentSign = signs[i];
                }
            }
        }

        if(currentSignPos == -1)
        {
            break;
        }

        // Stock du nb
        tempNb = stringToFloat(value.substr(0, currentSignPos));

        // Supression de l'inutil (nombre + signe)
        value.erase(0, currentSignPos + 1);

        // On prend la limite du prochain nombre
        currentSignPos = -1;
        for(int i = 0; i < tblSize; i++)
        {
                if(currentSignPos == -1 || currentSignPos > value.find(signs[i], 1)) // Si premier signe trouvé ou si ce signe est plus proche du début que celui d'avant : il faut le prendre.
                {
                    currentSignPos = value.find(signs[i], 1);
                }
        }

        // Si le prochain nombre est le dernier de la chaine
        if(currentSignPos == -1)
        {
            currentSignPos = value.size();
        }


        // On calcul
        if(currentSign == '+')
            tempNb += stringToFloat(value.substr(0, currentSignPos));
        else if(currentSign == '-')
            tempNb -= stringToFloat(value.substr(0, currentSignPos));
        else if(currentSign == '*')
            tempNb *= stringToFloat(value.substr(0, currentSignPos));
        else if(currentSign == '/')
            tempNb /= stringToFloat(value.substr(0, currentSignPos));

        // On supprime l'encien nombre MAIS EN LAISSANT LE PROCHAIN SIGNE !
        value.erase(0, currentSignPos);

        // On réinssert le resultat
        value.insert(0, floatToString(tempNb));
    }

    return value;
}
//------------------------------------------------------------------------------
std::string RcManager::ParseStringInfo(std::string value)
{
    // Etape 1 : Les variables
    int varBeginPos = 0;
    int varEndPos = 0;

    while(value.find('&') != std::string::npos) // Si debut de variable...
    {
        varBeginPos = value.find('&');
        if(value.find('&', varBeginPos + 1) != std::string::npos) // Si fin de variable ...
        {
            varEndPos = value.find('&', varBeginPos + 1);
            std::string varName = value.substr(varBeginPos + 1, varEndPos - varBeginPos - 1);
            // On supprime la variable de la chaine...
            value.erase(varBeginPos, varEndPos - varBeginPos + 1);
            // On ajoute la bonne valeur...
            value.insert(varBeginPos, GetValueStr(varName));
        }
        else
        {
            ERROR_FLUX << "Not ended variable !\nWill now exit." << std::endl;
            exit (-1);
        }
    }


    // Etape 2 : Calcul
    if(value[0] == '+') // Si + au début ==> Calcul
    {
        // On enléve le Plus
        value.erase(0,1);

        value = Calcl(value);
    }

    return value;
}

//------------------------------------------------------------------------------
float RcManager::GetValue(const std::string &id)
{
    if(m_i.m_ValueMap.find(id) != m_i.m_ValueMap.end())
    {
        return stringToFloat(ParseStringInfo(m_i.m_ValueMap[id]));
    }
    else
    {
        ERROR_FLUX << "Can't find \"" << id << "\" in info db !\nWill now exit." << std::endl;
        exit (-1);
    }
}
//------------------------------------------------------------------------------

std::string RcManager::GetValueStr(const std::string &id)
{
    if(m_i.m_ValueMap.find(id) != m_i.m_ValueMap.end())
    {
        return ParseStringInfo(m_i.m_ValueMap[id]);
    }
    else
    {
        ERROR_FLUX << "Can't find \"" << id << "\" in info db !\nWill now exit." << std::endl;
        exit (-1);
    }
}
//------------------------------------------------------------------------------
void RcManager::NewGame()
{
    GetInfo(m_i, m_i.m_modeDir + std::string("/mode-info.conf"));
}

//------------------------------------------------------------------------------
void LoadNewModeThread(void* UserData)
{
    StructAllInfo &m_i = *static_cast<StructAllInfo*>(UserData);

    m_i.m_loadProgress = 0;
    m_i.m_ImgMap.clear();
    m_i.m_SndBufMap.clear();
    m_i.m_FontMap.clear();
    m_i.m_FilesList.clear();
    m_i.m_ValueMap.clear();
    m_i.m_EnemyStockList.clear();

    GetInfo(m_i, m_i.m_modeDir + std::string("/mode-info.conf"));
    GetFileListFromNormalFile(m_i, m_i.m_modeDir + std::string("/mode-files_base.conf"));
    GetEnemies(m_i, m_i.m_modeDir + std::string("/mode-enemies.conf"));

    m_i.m_ImgMap["no"]; // Image vide.

    for(int i = 0; i < m_i.m_FilesList.size(); i++)
    {
        AddFile(m_i, m_i.m_modeDir, m_i.m_FilesList[i].file, m_i.m_FilesList[i].alias);
        m_i.m_loadProgress = float(i) / float(m_i.m_FilesList.size()) * 100;
    }

    if(m_i.m_musicOn)
    {
        m_i.m_MusicAmbience.Play();
    }

    m_i.m_loadProgress = 100;
}

//------------------------------------------------------------------------------
void RcManager::LoadNewMode(std::string modeDir)
{
    m_i.m_loadProgress = 0;
    m_i.m_modeDir = modeDir;

    #if defined NO_LOAD_THREAD
    LoadNewModeThread(&m_i);
    #else
    // Création d'un thread avec notre fonction
    sf::Thread *Thread = new sf::Thread(&LoadNewModeThread, &m_i);

    // Lancement du thread !
    Thread->Launch();
    #endif
}
//------------------------------------------------------------------------------
/*void RcManager::ReloadBackground()
{
    int newLevel = stringToInt(ParseStringInfo(m_i.m_ValueMap["LEVEL"]));

    if(newLevel != m_i.m_currentBackgroundLevel) // Si le level a changé depuis la derniére fois...
    {
        // On enléve le vieux background
        if(ImageExists(std::string("game-background(") + intToString(m_i.m_currentBackgroundLevel) + std::string(")")))
        {
            // On suprimme le background de l'ancien level :
            m_i.m_ImgMap.erase(std::string("game-background(") + intToString(m_i.m_currentBackgroundLevel) + std::string(")"));
        }

        if(newLevel == m_i.m_currentBackgroundLevel + 1) // Cas 1 : Level monte.
        {
           // On charge le suivant
           // qdsf
        }
        else
        {
            // On enléve l'ancien suivant
            // qds

            // On charge le current background
            if(m_i.m_FilesList.find(std::string("game-background(") + intToString(newLevel) + std::string(")")))
            {
                AddFile(m_i, m_i.m_mode, m_i.m_FilesList(std::string("game-background(") + intToString(newLevel) + std::string(")")), std::string("game-background(") + intToString(newLevel) + std::string(")"));
            }
        }

       // On charge le suivant
       // qdsf

        m_i.m_currentBackgroundLevel = newLevel;
    }
}*/
//------------------------------------------------------------------------------
void AddFile(StructAllInfo &m_i, const std::string &modeDir, const std::string &realName, const std::string &alias)
{
    std::string completePath = modeDir + std::string("/") + realName;

    std::string nameOnly = realName;

    if(nameOnly.rfind('/') != -1) // Si dans un sous dossier ==> On extrat le nom seul.
    {
        nameOnly = nameOnly.substr(nameOnly.rfind('/') + 1);
    }

    INFO_FLUX << "Loading \"" << realName << "\" as \"" << alias << "\"...";


    if(nameOnly.substr(0, 3) == std::string("img"))
    {
        if(m_i.m_ImgMap.find(alias) != m_i.m_ImgMap.end())
        {
            INFO_FLUX << "[Already loaded]" << std::endl;
            return;
        }
        else if(nameOnly == "img_no")
        {
            INFO_FLUX << "[No]" << std::endl;
            m_i.m_ImgMap[alias]; // On ajoute une image vide.
            return;
        }

        if(!m_i.m_ImgMap[alias].LoadFromFile(completePath))
        {
            ERROR_FLUX << "Can't load \"" << completePath << "\" !\nWill now exit." << std::endl;
            exit (-1);
        }
    }
    else if(nameOnly.substr(0, 3) == std::string("snd"))
    {
        if(m_i.m_SndBufMap.find(alias) != m_i.m_SndBufMap.end())
        {
            INFO_FLUX << "[Already loaded]" << std::endl;
            return;
        }

        if(!m_i.m_SndBufMap[alias].LoadFromFile(completePath))
        {
            ERROR_FLUX << "Can't load \"" << completePath << "\" !\nWill now exit." << std::endl;
            exit (-1);
        }
    }
    else if(nameOnly.substr(0, 4) == std::string("font"))
    {
        if(m_i.m_FontMap.find(alias) != m_i.m_FontMap.end())
        {
            INFO_FLUX << "[Already loaded]" << std::endl;
            return;
        }

        if(!m_i.m_FontMap[alias].LoadFromFile(completePath))
        {
            ERROR_FLUX << "Can't load \"" << completePath << "\" !\nWill now exit." << std::endl;
            exit (-1);
        }
    }
    else if(nameOnly.substr(0, 5) == std::string("music"))
    {
        if(!m_i.m_MusicAmbience.OpenFromFile(completePath))
        {
            ERROR_FLUX << "Can't load \"" << completePath << "\" !\nWill now exit." << std::endl;
            exit (-1);
        }
        m_i.m_MusicAmbience.SetLoop(true);
        m_i.m_MusicAmbience.Stop();
    }
    else
    {
        ERROR_FLUX << "Can't load \"" << completePath << "\"  : Unknown file !\nWill now exit." << std::endl;
        exit (-1);
    }

    INFO_FLUX << "[OK]" << std::endl;
}
//------------------------------------------------------------------------------
void RcManager::SetMusicOn(const bool &musicOn)
{
    m_i.m_musicOn = musicOn;
    if(IsMusicOn() && musicOn == false)
    {
        m_i.m_MusicAmbience.Stop();
    }
    else if(!IsMusicOn() && musicOn == true)
    {
        m_i.m_MusicAmbience.Play();
    }
}
//------------------------------------------------------------------------------
bool RcManager::IsMusicOn()
{
    if(m_i.m_MusicAmbience.GetStatus() == sf::Sound::Playing)
    {
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
void RcManager::SetSoundOn(const bool &soundOn)
{
    m_i.m_soundOn = soundOn;
}
//------------------------------------------------------------------------------
bool RcManager::IsSoundOn()
{
    return m_i.m_soundOn;
}
//------------------------------------------------------------------------------
float RcManager::GetLoadProgress() const
{
    return m_i.m_loadProgress;
}

//------------------------------------------------------------------------------
bool RcManager::PlaySnd(const std::string &name)
{
    if(name == std::string("no") || name.empty())
    {
        return false;
    }

    if(!m_i.m_soundOn)
    {
        return false; // Son désactivé.
    }

    for(int i = 0; i < MAX_SIMULTANEOUS_SND; i++)
    {
        if(m_Snd[i].GetStatus() == sf::Sound::Stopped)
        {
            m_Snd[i].SetBuffer(GetSoundBuffer(name));
            m_Snd[i].Play();
            return true;
        }
    }
    return false;
}
//------------------------------------------------------------------------------

sf::Image &RcManager::GetImage(const std::string &name)
{
    if(m_i.m_ImgMap.find(name) != m_i.m_ImgMap.end())
    {
        return m_i.m_ImgMap[name];
    }
    else
    {
        ERROR_FLUX << "Can't find \"" << name << "\" in file db !\nWill now exit." << std::endl;
        exit (-1);
    }
}
//------------------------------------------------------------------------------
bool RcManager::ImageExists(const std::string &name)
{
    if(m_i.m_ImgMap.find(name) != m_i.m_ImgMap.end())
    {
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
sf::Font &RcManager::GetFont(const std::string &name)
{
    if(m_i.m_FontMap.find(name) != m_i.m_FontMap.end())
    {
        return m_i.m_FontMap[name];
    }
    else
    {
        ERROR_FLUX << "Can't find \"" << name << "\" in file db !\nWill now exit." << std::endl;
        exit (-1);
    }
}
//------------------------------------------------------------------------------

sf::SoundBuffer &RcManager::GetSoundBuffer(const std::string &name)
{
    if(m_i.m_SndBufMap.find(name) != m_i.m_SndBufMap.end())
    {
        return m_i.m_SndBufMap[name];
    }
    else
    {
        ERROR_FLUX << "Can't find \"" << name << "\" in file db !\nWill now exit." << std::endl;
        exit (-1);
    }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
std::string intToString(int nb)
{
    std::stringstream ss;
    std::string str;
    ss << nb;
    ss >> str;
    return str;
}

//------------------------------------------------------------------------------
int stringToInt(std::string nbStr)
{
    std::stringstream ss;
    int nb;
    ss << nbStr;
    ss >> nb;
    return nb;
}
//------------------------------------------------------------------------------

float stringToFloat(std::string nbStr)
{
    std::stringstream ss;
    double nb;
    ss << nbStr;
    ss >> nb;
    return nb;
}
//------------------------------------------------------------------------------

std::string floatToString(float nb)
{
    std::stringstream ss;
    std::string str;
    ss << nb;
    ss >> str;
    return str;
}
